﻿using DocumentFormat.OpenXml.Bibliography;
using InternetShopContracts.DataBindingModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.LogicsContracts;
using System.Text.RegularExpressions;

namespace InternetShopForms.Orders
{
    public partial class FormOrderEdit : Form
    {
        private IOrderLogic _orderLogic;
        private IProductLogic _productLogic;
        public int? OrderId;
        private bool IsFieldsChanged = false;
        private bool IsFieldsSaved = false;
        private string OriginalTitle;
        private bool IsFormLoaded = false;
        public FormOrderEdit(IOrderLogic orderLogic, IProductLogic productLogic)
        {
            InitializeComponent();
            _orderLogic = orderLogic;
            OriginalTitle = Text;
            writeComponentCustomerEmail.ChangeText += textBoxCustomerEmail_Changed;
            productsList.CheckedItemChanged += listBoxProductsList_Changed;
            _productLogic = productLogic;
            productsList.MaxCheckedItemsCount = 1000;
            writeComponentCustomerEmail.Template = new Regex(@"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$");
        }

        private void FieldChanged()
        {
            if (IsFormLoaded)
            {
                if (!IsFieldsChanged)
                {
                    Text = OriginalTitle + "*";
                }
                IsFieldsChanged = true;
            }
        }

        private void buttonSelectImage_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.InitialDirectory = "d:\\tmp";
                openFileDialog.Filter = "Image files (*.jpg;*.png)|*.jpg;*.png|All files (*.*)|*.*";
                openFileDialog.FilterIndex = 1;
                openFileDialog.RestoreDirectory = true;

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    string prevText = textBoxImagePath.Text;
                    textBoxImagePath.Text = openFileDialog.FileName;
                    if (prevText != textBoxImagePath.Text)
                    {
                        FieldChanged();
                    }
                }
            }
        }

        private void FormOrderEdit_Load(object sender, EventArgs e)
        {
            LoadProductsList();
            if (OrderId != null)
            {
                LoadOrderData();
            }
            IsFormLoaded = true;
        }

        private void LoadProductsList()
        {
            var products = _productLogic.ReadList();
            productsList.SetCheckedListBoxValues(products.Select(x => x.Name).ToList());
        }

        private void LoadOrderData()
        {
            OrderViewModel? orderViewModel = _orderLogic.ReadElement(
                new InternetShopContracts.DataSearchModels.OrderSearchModel { Id = OrderId }
            );
            if (orderViewModel == null)
            {
                MessageBox.Show("Заказ не найден", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            textBoxCustomerFIO.Text = orderViewModel.CustomerFIO;
            textBoxImagePath.Text = orderViewModel.ImagePath;
            writeComponentCustomerEmail.WriteTextInTextBox = orderViewModel.CustomerEmail;
            productsList.CheckedItems = orderViewModel.ProductNames;
        }

        private void FormOrderEdit_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (IsFieldsChanged && !IsFieldsSaved)
            {
                DialogResult result = MessageBox.Show("Изменения не сохранятся, вы хотите выйти?",
                    "Предупреждение", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (result == DialogResult.No)
                {
                    e.Cancel = true;
                }
            }
        }

        private void textBoxCustomerFIO_TextChanged(object sender, EventArgs e)
        {
            FieldChanged();
        }

        private void textBoxImagePath_TextChanged(object sender, EventArgs e)
        {
            FieldChanged();
        }

        private void listBoxProductsList_Changed(object sender, EventArgs e)
        {
            FieldChanged();
        }

        private void textBoxCustomerEmail_Changed(object sender, EventArgs e)
        {
            FieldChanged();
        }

        private void buttonAccept_Click(object sender, EventArgs e)
        {
            OrderBindingModel? orderBindingModel = new OrderBindingModel();
            if (OrderId == null)
            {
                orderBindingModel = new OrderBindingModel();
            }
            else
            {
                var viewModel = _orderLogic.ReadElement(new InternetShopContracts.DataSearchModels.OrderSearchModel { Id = OrderId });
                if (viewModel == null)
                {
                    throw new Exception("Заказ не найден");
                }
                orderBindingModel = OrderBindingModel.FromViewModel(viewModel);
            }

            orderBindingModel.CustomerFIO = textBoxCustomerFIO.Text;
            try
            {
                orderBindingModel.CustomerEmail = writeComponentCustomerEmail.WriteTextInTextBox;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Проверьте поле \"Email заказчика\":\n" + ex.Message);
                return;
            }
            orderBindingModel.ImagePath = textBoxImagePath.Text;
            orderBindingModel.ProductNames = productsList.CheckedItems;

            try
            {
                bool savedOK;
                if (OrderId == null)
                {
                    savedOK = _orderLogic.Create(orderBindingModel);
                }
                else
                {
                    savedOK = _orderLogic.Update(orderBindingModel);
                }
                if (savedOK)
                {
                    DialogResult = DialogResult.OK;
                    IsFieldsSaved = true;
                    Close();
                }
                else
                {
                    throw new Exception("Не удалось сохранить заказ");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Произошла ошибка при сохранении:\n" + ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
